SystemOrganization addCategory: #'WebClient-Pharo'!

!Collection methodsFor: '*WebClient-Pharo-override' stamp: 'ar 5/5/2010 20:10'!
ifEmpty: aBlock
	"Evaluate the block if I'm empty"

	self isEmpty ifTrue: [ ^aBlock value ]! !

!ReadStream methodsFor: '*WebClient-Pharo-override' stamp: 'ar 7/30/2010 21:09'!
readInto: aCollection startingAt: startIndex count: n
	"Read n objects into the given collection. 
	Return number of elements that have been read."
	| max |
	max := (readLimit - position) min: n.
	aCollection 
		replaceFrom: startIndex 
		to: startIndex+max-1
		with: collection
		startingAt: position+1.
	position := position + max.
	^max! !

!SocketStream methodsFor: '*WebClient-Pharo-override' stamp: 'ar 5/4/2010 23:51'!
next: anInteger into: aCollection startingAt: startIndex
	"Read n objects into the given collection. 
	Return aCollection or a partial copy if less than
	n elements have been read."

	"Implementation note: This method DOES signal timeout if not 
	enough elements are received. It does NOT signal
	ConnectionClosed as closing the connection is the only way by
	which partial data can be read."

	| start amount |

	[self receiveData: anInteger] on: ConnectionClosed do:[:ex| ex return].

	"Inlined version of nextInBuffer: to avoid copying the contents"
	amount := anInteger min: (inNextToWrite - lastRead - 1).
	start := lastRead + 1.
	lastRead := lastRead + amount.
	aCollection 
		replaceFrom: startIndex 
		to: startIndex + amount-1 
		with: inBuffer 
		startingAt: start.
	^amount < anInteger 
		ifTrue:[aCollection copyFrom: 1 to:  startIndex + amount-1]
		ifFalse:[aCollection]! !

!SocketStream methodsFor: '*WebClient-Pharo-override' stamp: 'ar 7/30/2010 21:16'!
next: n putAll: aCollection startingAt: startIndex
	"Put a String or a ByteArray onto the stream.
	Currently a large collection will allocate a large buffer.
	Warning: this does not work with WideString: they have to be converted first."

	self adjustOutBuffer: n.
	outBuffer replaceFrom: outNextToWrite to: outNextToWrite + n - 1 with: aCollection startingAt: startIndex.
	outNextToWrite := outNextToWrite + n.
	self checkFlush.
	^aCollection! !

!SocketStream methodsFor: '*WebClient-Pharo-override' stamp: 'ar 7/30/2010 21:17'!
nextInto: aCollection
	"Read the next elements of the receiver into aCollection.
	Return aCollection or a partial copy if less than aCollection
	size elements have been read."
	^self next: aCollection size into: aCollection startingAt: 1.! !

!SocketStream methodsFor: '*WebClient-Pharo-override' stamp: 'igorStasenko 1/30/2010 13:52'!
peek
	"Return next byte, if inBuffer is empty
	we recieve some more data and try again.
	Do not consume the byte."

	[self isDataAvailable] whileFalse: [
		self receiveData. self atEnd ifTrue: [ ^ nil ]].
	
	^inBuffer at: lastRead +1! !

!SequenceableCollection class methodsFor: '*WebClient-Pharo-override' stamp: 'ar 5/5/2010 20:02'!
new: newSize streamContents: blockWithArg

	| stream |
	stream := WriteStream on: (self new: newSize).
	blockWithArg value: stream.
	stream position = newSize
		ifTrue: [ ^stream originalContents ]
		ifFalse: [ ^stream contents ]! !

!WriteStream methodsFor: '*WebClient-Pharo-override' stamp: 'ul 12/2/2009 05:47'!
crlf
	"Append a line feed character to the receiver."

	self nextPut: Character cr; nextPut: Character lf! !

Notification subclass: #HTTPProgress
	instanceVariableNames: 'total amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebClient-Pharo'!

!HTTPProgress commentStamp: 'ar 12/30/2009 16:16' prior: 0!
HTTP progress notification. Includes:
- total: The total size of the download (if known)
- amount: The completed amount of the download (if known)
!

!HTTPProgress methodsFor: 'accessing' stamp: 'ar 12/30/2009 16:15'!
amount
	"Answer the completed amount of the download (if known)"
	^amount! !

!HTTPProgress methodsFor: 'accessing' stamp: 'ar 12/30/2009 16:15'!
amount: bytes
	"Set the completed amount of the download (if known)"
	amount := bytes! !

!HTTPProgress methodsFor: 'accessing' stamp: 'ar 12/30/2009 16:15'!
total
	"Answer the total size of the download, if known"
	^total! !

!HTTPProgress methodsFor: 'accessing' stamp: 'ar 12/30/2009 16:15'!
total: bytes
	"Answer the total size of the download, if known"
	total := bytes! !

!String methodsFor: '*WebClient-Pharo-override' stamp: 'ar 5/4/2010 23:48'!
, anObject
	"Concatenate the argument to the receiver."

	^ self copyReplaceFrom: self size + 1
		  to: self size
		  with: anObject asString! !

!String methodsFor: '*WebClient-Pharo-override' stamp: 'ar 5/4/2010 23:52'!
squeakToUtf8
	"Convert the receiver into a UTF8-encoded string"
	
	^self convertToWithConverter: UTF8TextConverter new.! !

!String methodsFor: '*WebClient-Pharo-override' stamp: 'ar 7/30/2010 20:44'!
utf8ToSqueak
	"Convert the receiver from a UTF8-encoded string"
	
	^self convertFromWithConverter: UTF8TextConverter new.! !
